/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___TextEffects___
#define ___TextEffects___

#include <LibGenerator/glew.h>
#include <wx/image.h>
#include <wx/glcanvas.h>
#include <LibGenerator/ProceduralGenerator.h>
#include <LibGenerator/GlObjects.h>

namespace ProceduralTextures
{
	class TextEffects;
	class TePlugin;
	class TePanel;

	typedef enum
	{	eBG_TYPE_COLOUR
	,	eBG_TYPE_TEXTURE
	}	eBG_TYPE;

	class TextEffects : public ProceduralGenerator
	{
	private:
		class Glyph
		{
		private:
			wxPoint				m_ptPosition;
			wxSize				m_size;
			std::vector<byte>	m_bitmap;

		public:
			Glyph();
			Glyph( const Glyph & p_glyph);
			Glyph & operator =( const Glyph & p_glyph);
			~Glyph();

			inline const wxSize &				GetSize		()const { return m_size; }
			inline const wxPoint &				GetPosition	()const { return m_ptPosition; }
			inline const std::vector<byte> &	GetBitmap	()const { return m_bitmap; }

			inline void	SetBitmap	( const std::vector<byte> & val)	{ m_bitmap = val; }
			inline void SetPosition	( const wxPoint & val)				{ m_ptPosition = val; }
			inline void SetSize		( const wxSize & val)				{ m_size = val; }
		};

	private:
		wxImage					m_image;
		int						m_iImgWidth;
		int						m_iImgHeight;
		wxString				m_strText;
		wxString				m_strFontFile;
		bool					m_bFontModified;
		bool					m_bTextModified;
		std::vector<Glyph>		m_arrayChars;
		int						m_iMaxHeight;
		int						m_iMaxTop;
		GlTexture				m_textureImg;
		GlTexture				m_textureTmpImg;
		GlTexture				m_textureTxtImg;
		GlTexture				m_textureTxt;
		GlTexture				m_textureTmpTxt;
		GlFrameBuffer			m_frameBufferImg;
		GlFrameBuffer			m_frameBufferTxt;
		bool					m_bNewTexture;
		bool					m_bNewTxtTexture;
		UbPixel					m_txtColour;
		wxImage					m_txtImage;
		GlTexture::BlendMode	m_blendRgbTxt;
		GlTexture::BlendMode	m_blendAlphaTxt;
		GlTexture::BlendMode	m_blendRgbImg;
		GlTexture::BlendMode	m_blendAlphaImg;
		eBG_TYPE				m_eTextBackgroundType;

	public:
		TextEffects( int p_width, int p_height, int p_iWndId, wxFrame * p_pFrame);
		virtual ~TextEffects();

		void SetImagePath( const wxString & p_strImagePath);
		void SetTxtColour( const wxColour & p_clrColour);
		void SetTxtImagePath( const wxString & p_strImagePath);
		void SetTextBackgroundType( eBG_TYPE val);

		inline void SetFont					( const wxString & p_strFont)	{ m_strFontFile = p_strFont;m_bFontModified = true; }
		inline void	SetText					( const wxString & p_strText)	{ m_strText = p_strText;m_bTextModified = true; }

	private:
		void _loadFtFont();
		void _cleanupArrays();
		void _updateTxtImage();

		virtual void _resize		( const wxSize & p_size);
		virtual void _glInit		();
		virtual void _glPreRender	();
		virtual void _glRender		( bool & p_bChanged);
		virtual void _glPostRender	();
		virtual void _glCleanup		();
	};

	class TePanel : public ConfigPanel
	{
	private:
		typedef enum
		{	eTextBgType	= 51
		,	eTextBg
		,	eFont
		,	eText
		,	eBackground
		}	eIDs;

		TextEffects				*	m_pGenerator;
		wxComboBox				*	m_pComboTextBackground;
		wxButton				*	m_pButtonColour;
		wxButton				*	m_pButtonBackground;
		wxButton				*	m_pButtonFont;
		wxTextCtrl				*	m_pTextCaption;

	public:
		TePanel( wxWindow * p_pWindow, wxWindowID p_id, const wxPoint & p_ptPosition=wxDefaultPosition, const wxSize & p_size=wxDefaultSize);
		virtual ~TePanel();

		void Create( TextEffects * p_pGenerator, wxWindow * p_pWindow, const wxPoint & p_ptPosition, const wxSize & p_size);

	private:
		DECLARE_EVENT_TABLE();
		void _onSize				( wxSizeEvent		&	p_event);
		void _onBackground			( wxCommandEvent	&	p_event);
		void _onTxtBackgroundType	( wxCommandEvent	&	p_event);
		void _onTxtBackground		( wxCommandEvent	&	p_event);
		void _onFont				( wxCommandEvent	&	p_event);
		void _onChar				( wxCommandEvent	&	p_event);
	};

	class TePlugin : public Plugin<TePanel, TextEffects, TePlugin>
	{
	public:
		TePlugin();
		virtual ~TePlugin();

		virtual wxString GetName();

		DECLARE_PLUGIN()
	};
}

#endif
