#include "IteratedFunctionSystem/GeneratorIteratedFunctionSystem.h"
#include <LibGenerator/Random.h>

#ifdef __WXMSW__
#	include<wx/msw/msvcrt.h>
#endif

using namespace ProceduralTextures;

//*************************************************************************************************

IfsCell :: IfsCell()
	:	m_pixel( NULL)
{
}

IfsCell :: IfsCell( const IfsCell & cell)
	:	m_pixel( cell.m_pixel)
	,	m_ptPosition( cell.m_ptPosition)
{
}

IfsCell :: ~IfsCell()
{
}

IfsCell & IfsCell :: operator =( const IfsCell & cell)
{
	m_pixel = cell.m_pixel;
	m_ptPosition = cell.m_ptPosition;

	return * this;
}

void IfsCell :: Set( UbPixel * p_pixel, const Point<double, 2> & p_ptPosition)
{
	m_pixel = p_pixel;
	m_ptPosition = p_ptPosition;
}

//*************************************************************************************************

IteratedFunctionSystem::IfsGenerator :: IfsGenerator( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight, int p_iAngle, int p_iShift, int p_iSize, const Point<double, 2> & p_ptPosition, size_t p_uiVariationCount)
	:	m_pBackBuffer( p_pBackBuffer)
	,	m_iHeight( p_iHeight)
	,	m_iWidth( p_iWidth)
	,	m_iAngle( p_iAngle)
	,	m_iShift( p_iShift)
	,	m_iSize(p_iSize)
	,	m_ptPositionInSection( p_ptPosition)
	,	m_uiVariationCount( p_uiVariationCount)
	,	m_pxColour( 16, 16, 16, 255)
{
	m_arrayVariations.resize( m_uiVariationCount * 2);
	m_arrayProbabilities.resize( m_uiVariationCount);
}

IteratedFunctionSystem::IfsGenerator :: ~IfsGenerator()
{
}

void IteratedFunctionSystem::IfsGenerator :: Initialise( IfsCellMap & p_arrayGeneratedCellsMap, IfsCellLst & p_arrayGeneratedCellsLst, IfsCellVct & p_arrayGeneratedCellsVct)
{
	m_arrayInitCellsMap.clear();
	Point<double, 2> l_ptPosition;
	size_t l_iCount = 0;
	m_arrayInitCellsVct.reserve( m_uiVariationCount);

	for (size_t k = 0 ; k < m_uiVariationCount ; k++)
	{
		l_ptPosition[0] = m_arrayVariations[l_iCount++][2];
		l_ptPosition[1] = m_arrayVariations[l_iCount++][2];
		IfsCell l_cell;

		if (_insertCell( l_ptPosition, p_arrayGeneratedCellsMap, & l_cell))
		{
			m_arrayInitCellsMap.push_back( p_arrayGeneratedCellsMap);
			p_arrayGeneratedCellsLst.push_back( l_cell);
			m_arrayInitCellsLst.push_back( p_arrayGeneratedCellsLst);
			p_arrayGeneratedCellsVct.push_back( l_cell);
			m_arrayInitCellsVct.push_back( p_arrayGeneratedCellsVct);
		}
	}
}

bool IteratedFunctionSystem::IfsGenerator :: Generate( const IfsCell & p_cell, IfsCellMap & p_arrayGeneratedCells)
{
	bool l_bReturn = false;

	Point<double, 2> l_ptPosition;
	size_t l_iCount = 0;
	int iProbability = Random::rand() % 100;

	for (size_t k = 0 ; k < m_uiVariationCount ; k++)
	{
		if (iProbability >= m_arrayProbabilities[k][0] && iProbability < m_arrayProbabilities[k][1])
		{
			l_ptPosition = p_cell.m_ptPosition;
			l_ptPosition[0] = l_ptPosition[0] * m_arrayVariations[l_iCount][0] + l_ptPosition[1] * m_arrayVariations[l_iCount][1] + m_arrayVariations[l_iCount][2];l_iCount++;
			l_ptPosition[1] = l_ptPosition[0] * m_arrayVariations[l_iCount][0] + l_ptPosition[1] * m_arrayVariations[l_iCount][1] + m_arrayVariations[l_iCount][2];l_iCount++;
			l_bReturn |= _insertCell( l_ptPosition, p_arrayGeneratedCells);
		}
		else
		{
			l_iCount++;
			l_iCount++;
		}
	}

	return l_bReturn;
}

bool IteratedFunctionSystem::IfsGenerator :: Generate( const IfsCell & p_cell, IfsCellLst & p_arrayGeneratedCells)
{
	bool l_bReturn = false;

	Point<double, 2> l_ptPosition;
	size_t l_iCount = 0;
	int iProbability = Random::rand() % 100;

	for (size_t k = 0 ; k < m_uiVariationCount ; k++)
	{
		if (iProbability >= m_arrayProbabilities[k][0] && iProbability < m_arrayProbabilities[k][1])
		{
			l_ptPosition = p_cell.m_ptPosition;
			l_ptPosition[0] = l_ptPosition[0] * m_arrayVariations[l_iCount][0] + l_ptPosition[1] * m_arrayVariations[l_iCount][1] + m_arrayVariations[l_iCount][2];l_iCount++;
			l_ptPosition[1] = l_ptPosition[0] * m_arrayVariations[l_iCount][0] + l_ptPosition[1] * m_arrayVariations[l_iCount][1] + m_arrayVariations[l_iCount][2];l_iCount++;
			l_bReturn |= _insertCell( l_ptPosition, p_arrayGeneratedCells);
		}
		else
		{
			l_iCount++;
			l_iCount++;
		}
	}

	return l_bReturn;
}

bool IteratedFunctionSystem::IfsGenerator :: Generate( IfsCell p_cell, IfsCellVct & p_arrayGeneratedCells)
{
	bool l_bReturn = false;

	Point<double, 2> l_ptPosition;
	size_t l_iCount = 0;
	int iProbability = Random::rand() % 100;

	for (size_t k = 0 ; k < m_uiVariationCount ; k++)
	{
		if (iProbability >= m_arrayProbabilities[k][0] && iProbability < m_arrayProbabilities[k][1])
		{
			l_ptPosition = p_cell.m_ptPosition;
			l_ptPosition[0] = l_ptPosition[0] * m_arrayVariations[l_iCount][0] + l_ptPosition[1] * m_arrayVariations[l_iCount][1] + m_arrayVariations[l_iCount][2];l_iCount++;
			l_ptPosition[1] = l_ptPosition[0] * m_arrayVariations[l_iCount][0] + l_ptPosition[1] * m_arrayVariations[l_iCount][1] + m_arrayVariations[l_iCount][2];l_iCount++;
			l_bReturn |= _insertCell( l_ptPosition, p_arrayGeneratedCells);
		}
		else
		{
			l_iCount++;
			l_iCount++;
		}
	}

	return l_bReturn;
}

void IteratedFunctionSystem::IfsGenerator :: Resample()
{
}

bool IteratedFunctionSystem::IfsGenerator :: NextGeneration()
{
	if (m_pxColour.r < 239)
	{
		m_pxColour += UbPixel( 16, 16, 16, 255);
	}
	return true;
}

bool IteratedFunctionSystem::IfsGenerator :: _insertCell( const Point<double, 2> & p_ptPosition, IfsCellMap & p_arrayGeneratedCells, IfsCell * p_pResult)
{
	IfsCellMap::iterator l_it1;
	IfsCellIntMap::iterator l_it2;
	IfsCell l_cell;
	bool l_bReturn = false;

	int i = int( p_ptPosition[0] * m_iWidth);
	int j = int( p_ptPosition[1] * m_iHeight);
	m_ptMin[0] = std::min( i, m_ptMin[0]);
	m_ptMin[1] = std::min( j, m_ptMin[1]);
	m_ptMax[0] = std::max( i, m_ptMax[0]);
	m_ptMax[1] = std::max( j, m_ptMax[1]);

	l_it1 = p_arrayGeneratedCells.find( i);

	if (l_it1 == p_arrayGeneratedCells.end())
	{
		p_arrayGeneratedCells.insert( std::pair<int, std::map <int, IfsCell> >( i, std::map <int, IfsCell>()));
		l_it1 = p_arrayGeneratedCells.find( i);
	}

	l_it2 = l_it1->second.find( j);

	if (l_it2 == l_it1->second.end())
	{
		if (i >= 0 && j >= 0 && i < m_iWidth && j < m_iHeight)
		{
			l_cell.Set( & (*m_pBackBuffer)[i][j], p_ptPosition);
			l_cell.m_pixel->Set( m_pxColour.r, m_pxColour.g, m_pxColour.b, m_pxColour.a);
		}
		else
		{
			l_cell.Set( NULL, p_ptPosition);
		}

		l_it1->second.insert( std::pair<int, IfsCell>( j, l_cell));
		l_bReturn = true;

		if (p_pResult != NULL)
		{
			( * p_pResult) = l_cell;
		}
	}

	return l_bReturn;
}

bool IteratedFunctionSystem::IfsGenerator :: _insertCell( const Point<double, 2> & p_ptPosition, IfsCellLst & p_arrayGeneratedCells, IfsCell * p_pResult)
{
	IfsCell l_cell;
	bool l_bReturn = false;

	int i = int( p_ptPosition[0] * m_iWidth);
	int j = int( p_ptPosition[1] * m_iHeight);
	m_ptMin[0] = std::min( i, m_ptMin[0]);
	m_ptMin[1] = std::min( j, m_ptMin[1]);
	m_ptMax[0] = std::max( i, m_ptMax[0]);
	m_ptMax[1] = std::max( j, m_ptMax[1]);

	if (i >= 0 && j >= 0 && i < m_iWidth && j < m_iHeight)
	{
		l_cell.Set( & (*m_pBackBuffer)[i][j], p_ptPosition);
		l_cell.m_pixel->Set( m_pxColour.r, m_pxColour.g, m_pxColour.b, m_pxColour.a);
	}
	else
	{
		l_cell.Set( NULL, p_ptPosition);
	}

	p_arrayGeneratedCells.push_back( l_cell);
	l_bReturn = true;

	if (p_pResult != NULL)
	{
		( * p_pResult) = l_cell;
	}

	return l_bReturn;
}

bool IteratedFunctionSystem::IfsGenerator :: _insertCell( const Point<double, 2> & p_ptPosition, IfsCellVct & p_arrayGeneratedCells, IfsCell * p_pResult)
{
	IfsCell l_cell;
	bool l_bReturn = false;

	int i = int( p_ptPosition[0] * m_iWidth);
	int j = int( p_ptPosition[1] * m_iHeight);
	m_ptMin[0] = std::min( i, m_ptMin[0]);
	m_ptMin[1] = std::min( j, m_ptMin[1]);
	m_ptMax[0] = std::max( i, m_ptMax[0]);
	m_ptMax[1] = std::max( j, m_ptMax[1]);

	if (i >= 0 && j >= 0 && i < m_iWidth && j < m_iHeight)
	{
		l_cell.Set( & (*m_pBackBuffer)[i][j], p_ptPosition);
		l_cell.m_pixel->Set( m_pxColour.r, m_pxColour.g, m_pxColour.b, m_pxColour.a);
	}
	else
	{
		l_cell.Set( NULL, p_ptPosition);
	}

	p_arrayGeneratedCells.push_back( l_cell);
	l_bReturn = true;

	if (p_pResult != NULL)
	{
		( * p_pResult) = l_cell;
	}

	return l_bReturn;
}

//*************************************************************************************************

IteratedFunctionSystem::SierpinskiGasket :: SierpinskiGasket( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 50, 50, 50, Point<double, 2>( 0, 0), 3)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::SierpinskiGasket :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.500,  0.000,  0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.000,  0.500,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 100);
	m_arrayVariations[2]	= Point<double, 3>(  0.500,  0.000,  0.500);		m_arrayVariations[3]	= Point<double, 3>(  0.000,  0.500,  0.000);	m_arrayProbabilities[1] = Point<int, 2>( 0, 100);
	m_arrayVariations[4]	= Point<double, 3>(  0.500,  0.000,  0.250);		m_arrayVariations[5]	= Point<double, 3>(  0.000,  0.500,  0.433);	m_arrayProbabilities[2] = Point<int, 2>( 0, 100);
}

//*************************************************************************************************

IteratedFunctionSystem::SierpinskiCarpet :: SierpinskiCarpet( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 50, 50, 50, Point<double, 2>( 0, 0), 8)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::SierpinskiCarpet :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.333,  0.000,  0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.000,  0.333,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 100);
	m_arrayVariations[2]	= Point<double, 3>(  0.333,  0.000,  0.000);		m_arrayVariations[3]	= Point<double, 3>(  0.000,  0.333,  0.333);	m_arrayProbabilities[1] = Point<int, 2>( 0, 100);
	m_arrayVariations[4]	= Point<double, 3>(  0.333,  0.000,  0.000);		m_arrayVariations[5]	= Point<double, 3>(  0.000,  0.333,  0.667);	m_arrayProbabilities[2] = Point<int, 2>( 0, 100);
	m_arrayVariations[6]	= Point<double, 3>(  0.333,  0.000,  0.333);		m_arrayVariations[7]	= Point<double, 3>(  0.000,  0.333,  0.000);	m_arrayProbabilities[3] = Point<int, 2>( 0, 100);
	m_arrayVariations[8]	= Point<double, 3>(  0.333,  0.000,  0.333);		m_arrayVariations[9]	= Point<double, 3>(  0.000,  0.333,  0.667);	m_arrayProbabilities[4] = Point<int, 2>( 0, 100);
	m_arrayVariations[10]	= Point<double, 3>(  0.333,  0.000,  0.667);		m_arrayVariations[11]	= Point<double, 3>(  0.000,  0.333,  0.000);	m_arrayProbabilities[5] = Point<int, 2>( 0, 100);
	m_arrayVariations[12]	= Point<double, 3>(  0.333,  0.000,  0.667);		m_arrayVariations[13]	= Point<double, 3>(  0.000,  0.333,  0.333);	m_arrayProbabilities[6] = Point<int, 2>( 0, 100);
	m_arrayVariations[14]	= Point<double, 3>(  0.333,  0.000,  0.667);		m_arrayVariations[15]	= Point<double, 3>(  0.000,  0.333,  0.667);	m_arrayProbabilities[7] = Point<int, 2>( 0, 100);
}

//*************************************************************************************************

IteratedFunctionSystem::FractalLeaf :: FractalLeaf( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 160, 4, 160, Point<double, 2>( 0.5, 0.0), 4)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::FractalLeaf :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.000,  0.000, 0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.000,  0.160,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 1);
	m_arrayVariations[2]	= Point<double, 3>(  0.850,  0.040, 0.000);		m_arrayVariations[3]	= Point<double, 3>( -0.040,  0.850,  1.600);	m_arrayProbabilities[1] = Point<int, 2>( 15, 100);
	m_arrayVariations[4]	= Point<double, 3>(  0.200, -0.260, 0.000);		m_arrayVariations[5]	= Point<double, 3>(  0.230,  0.220,  1.600);	m_arrayProbabilities[2] = Point<int, 2>( 1, 8);
	m_arrayVariations[6]	= Point<double, 3>( -0.150,  0.280, 0.000);		m_arrayVariations[7]	= Point<double, 3>(  0.260,  0.240,  0.440);	m_arrayProbabilities[3] = Point<int, 2>( 8, 15);
}

//*************************************************************************************************

IteratedFunctionSystem::HeighwayDragon :: HeighwayDragon( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 160, 4, 160, Point<double, 2>( 0.5, 0.0), 2)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::HeighwayDragon :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.500, -0.500,  0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.500,  0.500,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 100);
	m_arrayVariations[2]	= Point<double, 3>( -0.500, -0.500,  1.000);		m_arrayVariations[3]	= Point<double, 3>(  0.500, -0.500,  0.000);	m_arrayProbabilities[1] = Point<int, 2>( 0, 100);
}

//*************************************************************************************************

IteratedFunctionSystem::McWorterPentigree :: McWorterPentigree( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 160, 4, 160, Point<double, 2>( 0.5, 0.0), 6)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::McWorterPentigree :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.309, -0.205,  0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.225,  0.309,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 100);
	m_arrayVariations[2]	= Point<double, 3>( -0.118, -0.363,  0.309);		m_arrayVariations[3]	= Point<double, 3>(  0.363, -0.118,  0.225);	m_arrayProbabilities[1] = Point<int, 2>( 0, 100);
	m_arrayVariations[4]	= Point<double, 3>(  0.309,  0.225,  0.191);		m_arrayVariations[5]	= Point<double, 3>( -0.225,  0.309,  0.588);	m_arrayProbabilities[2] = Point<int, 2>( 0, 100);
	m_arrayVariations[6]	= Point<double, 3>( -0.118,  0.363,  0.500);		m_arrayVariations[7]	= Point<double, 3>( -0.363, -0.118,  0.363);	m_arrayProbabilities[3] = Point<int, 2>( 0, 100);
	m_arrayVariations[8]	= Point<double, 3>(  0.309,  0.225,  0.382);		m_arrayVariations[9]	= Point<double, 3>( -0.225,  0.309,  0.000);	m_arrayProbabilities[4] = Point<int, 2>( 0, 100);
	m_arrayVariations[10]	= Point<double, 3>(  0.309, -0.225,  0.691);		m_arrayVariations[11]	= Point<double, 3>(  0.225,  0.309, -0.225);	m_arrayProbabilities[5] = Point<int, 2>( 0, 100);
}

//*************************************************************************************************

IteratedFunctionSystem::Pentadentrite :: Pentadentrite( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 160, 4, 160, Point<double, 2>( 0.5, 0.0), 6)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::Pentadentrite :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.341, -0.071,  0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.071,  0.341,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 100);
	m_arrayVariations[2]	= Point<double, 3>(  0.038, -0.346,  0.341);		m_arrayVariations[3]	= Point<double, 3>(  0.346,  0.038,  0.071);	m_arrayProbabilities[1] = Point<int, 2>( 0, 100);
	m_arrayVariations[4]	= Point<double, 3>(  0.341, -0.071,  0.379);		m_arrayVariations[5]	= Point<double, 3>(  0.071,  0.341,  0.418);	m_arrayProbabilities[2] = Point<int, 2>( 0, 100);
	m_arrayVariations[6]	= Point<double, 3>( -0.234,  0.258,  0.720);		m_arrayVariations[7]	= Point<double, 3>( -0.258, -0.234,  0.489);	m_arrayProbabilities[3] = Point<int, 2>( 0, 100);
	m_arrayVariations[8]	= Point<double, 3>(  0.173,  0.302,  0.486);		m_arrayVariations[9]	= Point<double, 3>( -0.302,  0.173,  0.231);	m_arrayProbabilities[4] = Point<int, 2>( 0, 100);
	m_arrayVariations[10]	= Point<double, 3>(  0.341, -0.071,  0.659);		m_arrayVariations[11]	= Point<double, 3>(  0.071,  0.341, -0.071);	m_arrayProbabilities[5] = Point<int, 2>( 0, 100);
}

//*************************************************************************************************

IteratedFunctionSystem::KochCurve :: KochCurve( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 160, 4, 160, Point<double, 2>( 0.5, 0.0), 4)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::KochCurve :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.333,  0.000, 0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.000,  0.333,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 1);
	m_arrayVariations[2]	= Point<double, 3>(  0.167, -0.289, 0.333);		m_arrayVariations[3]	= Point<double, 3>(  0.289,  0.167,  0.000);	m_arrayProbabilities[1] = Point<int, 2>( 15, 100);
	m_arrayVariations[4]	= Point<double, 3>(  0.167,  0.289, 0.500);		m_arrayVariations[5]	= Point<double, 3>( -0.289,  0.167,  0.289);	m_arrayProbabilities[2] = Point<int, 2>( 1, 8);
	m_arrayVariations[6]	= Point<double, 3>(  0.333,  0.000, 0.667);		m_arrayVariations[7]	= Point<double, 3>(  0.000,  0.333,  0.000);	m_arrayProbabilities[3] = Point<int, 2>( 8, 15);
}

//*************************************************************************************************

IteratedFunctionSystem::LevyDragon :: LevyDragon( PixelBuffer * p_pBackBuffer, int p_iWidth, int p_iHeight)
	:	IfsGenerator( p_pBackBuffer, p_iWidth, p_iHeight, 160, 4, 160, Point<double, 2>( 0.5, 0.0), 2)
{
	_initialiseVariations();
}

void IteratedFunctionSystem::LevyDragon :: _initialiseVariations()
{
//							x = a[0]*x + a[1]*y + a[2]													y = b[0]*x + b[1]*y + b[2]
	m_arrayVariations[0]	= Point<double, 3>(  0.500, -0.500,  0.000);		m_arrayVariations[1]	= Point<double, 3>(  0.500,  0.500,  0.000);	m_arrayProbabilities[0] = Point<int, 2>( 0, 100);
	m_arrayVariations[2]	= Point<double, 3>(  0.500,  0.500,  0.500);		m_arrayVariations[3]	= Point<double, 3>( -0.500,  0.500,  0.500);	m_arrayProbabilities[1] = Point<int, 2>( 0, 100);
}

//*************************************************************************************************

IteratedFunctionSystem::Thread :: Thread( ProceduralGenerator * p_pParent, size_t p_uiIndex, int iWidth, int iTop, int iBottom, int iTotalHeight, const UbPixel & p_pxColour)
	:	ProceduralGenerator::Thread( p_pParent, p_uiIndex, iWidth, iTop, iBottom, iTotalHeight, p_pxColour)
{
}

IteratedFunctionSystem::Thread :: ~Thread()
{
	m_pMapCells = NULL;
	m_backBuffer = NULL;
}

void IteratedFunctionSystem::Thread :: Step()
{
//	StepLst();
//	StepVct();
	StepMap();
}

void IteratedFunctionSystem::Thread :: Initialise( IfsCellMap * p_pMapCells, IfsCellLst * p_pLstCells, IfsCellVct * p_pVctCells, PixelBuffer * p_pBackBuffer, IfsGenerator ** p_pGenerator)
{
	m_pMapCells = p_pMapCells;
	m_pLstCells = p_pLstCells;
	m_pVctCells = p_pVctCells;
	m_backBuffer = p_pBackBuffer;
	m_pGenerator = p_pGenerator;
	m_uiCells1 = 0;
	m_uiCellsSize = m_pVctCells->size();
	m_itLstCells1 = m_pLstCells->begin();
	m_itMapCells1 = m_pMapCells->begin();
	m_bResult = true;
}

void IteratedFunctionSystem::Thread :: StepMap()
{
	if (m_itMapCells1 != m_pMapCells->end())
	{
		for (IfsCellIntMap::iterator l_itMapCells2 = m_itMapCells1->second.begin() ; ! IsStopped() && l_itMapCells2 != m_itMapCells1->second.end() ; ++l_itMapCells2)
		{
			m_bResult |= (* m_pGenerator)->Generate( l_itMapCells2->second, m_mapGeneratedCells);
		}

		m_itMapCells1++;

		if (m_itMapCells1 == m_pMapCells->end())
		{
			if (m_bResult)
			{
				IfsCellMap::iterator l_itCells1;
				IfsCellIntMap::iterator l_itCells2;

				for (IfsCellMap::iterator l_it1 = m_mapGeneratedCells.begin() ; l_it1 != m_mapGeneratedCells.end() ; ++l_it1)
				{
					l_itCells1 = m_pMapCells->find( l_it1->first);

					if (l_itCells1 == m_pMapCells->end())
					{
						m_pMapCells->insert( IfsCellMap::value_type( l_it1->first, l_it1->second));
					}
					else
					{
						for (IfsCellIntMap::iterator l_it2 = l_it1->second.begin() ; l_it2 != l_it1->second.end() ; ++l_it2)
						{
							l_itCells2 = l_itCells1->second.find( l_it2->first);

							if (l_itCells2 == l_itCells1->second.end())
							{
								l_itCells1->second.insert( IfsCellIntMap::value_type( l_it2->first, l_it2->second));
							}
						}
					}
				}

				m_itMapCells1 = m_pMapCells->begin();

				m_mapGeneratedCells.clear();
				m_bResult = true;
				(* m_pGenerator)->NextGeneration();
			}
			else
			{
				Stop();
			}
		}
	}
	else
	{
		Stop();
	}
}

void IteratedFunctionSystem::Thread :: StepLst()
{
	if (m_itLstCells1 != m_pLstCells->end())
	{
		m_bResult |= (* m_pGenerator)->Generate( * m_itLstCells1, m_lstGeneratedCells);
		IfsCellLst::iterator l_itTmp = m_itLstCells1;
		m_itLstCells1++;

		if (m_itLstCells1 == m_pLstCells->end())
		{
			m_pLstCells->insert( l_itTmp, m_lstGeneratedCells.begin(), m_lstGeneratedCells.end());
			m_itLstCells1 = m_pLstCells->begin();
			(* m_pGenerator)->NextGeneration();
		}
	}
	else
	{
		Stop();
	}
}

void IteratedFunctionSystem::Thread :: StepVct()
{
	if (m_uiCells1 < m_pVctCells->size())
	{
		m_bResult |= (* m_pGenerator)->Generate( m_pVctCells->at( m_uiCells1), ( * m_pVctCells));
		m_uiCells1++;

		if (m_uiCells1 == m_uiCellsSize)
		{
			m_uiCells1 = 0;
			m_uiCellsSize = m_pVctCells->size();
			m_pVctCells->reserve( m_pVctCells->size() * (* m_pGenerator)->GetVariationCount());
			(* m_pGenerator)->NextGeneration();
		}
	}
	else
	{
		Stop();
	}
}

//*************************************************************************************************

wxString IteratedFunctionSystem::ArrayNames[eIFS_GENERATOR_COUNT];

IteratedFunctionSystem :: IteratedFunctionSystem( int p_width, int p_height, int p_iWndId, wxFrame * p_pFrame)
	:	ProceduralGenerator( p_width, p_height, p_iWndId, p_pFrame)
	,	m_pGenerator( NULL)
{
	for (int i = 0 ; i < eIFS_GENERATOR_COUNT ; i++)
	{
		switch (i)
		{
		case eIFS_GENERATOR_SIERPINSKI_CARPET:
			ArrayNames[i] = wxT( "Sierpinski Carpet");
			m_arrayGenerators[i] = new SierpinskiCarpet( & m_backBuffer, p_width, p_height);
			break;

		case eIFS_GENERATOR_SIERPINSKI_GASKET:
			ArrayNames[i] = wxT( "Sierpinski Gasket");
			m_arrayGenerators[i] = new SierpinskiGasket( & m_backBuffer, p_width, p_height);
			break;

		case eIFS_GENERATOR_FRACTAL_LEAF:
			ArrayNames[i] = wxT( "Fractal Leaf");
			m_arrayGenerators[i] = new FractalLeaf( & m_backBuffer, p_width, p_height);
			break;

		case eIFS_GENERATOR_HEIGHWAY_DRAGON:
			ArrayNames[i] = wxT( "Heighway Dragon");
			m_arrayGenerators[i] = new HeighwayDragon( & m_backBuffer, p_width, p_height);
			break;

		case eIFS_GENERATOR_MCWORTHER_PENTIGREE:
			ArrayNames[i] = wxT( "Mc Worther Pentigree");
			m_arrayGenerators[i] = new McWorterPentigree( & m_backBuffer, p_width, p_height);
			break;

		case eIFS_GENERATOR_PENTADENTRITE:
			ArrayNames[i] = wxT( "Pentadentrite");
			m_arrayGenerators[i] = new Pentadentrite( & m_backBuffer, p_width, p_height);
			break;

		case eIFS_GENERATOR_KOCH_CURVE:
			ArrayNames[i] = wxT( "Koch Curve");
			m_arrayGenerators[i] = new KochCurve( & m_backBuffer, p_width, p_height);
			break;

		case eIFS_GENERATOR_LEVY_DRAGON:
			ArrayNames[i] = wxT( "Levy Dragon");
			m_arrayGenerators[i] = new LevyDragon( & m_backBuffer, p_width, p_height);
			break;
		}
	}

	m_bNeedsConfigPanel = true;
	Random::srand( time( NULL));
	m_pGenerator = m_arrayGenerators[eIFS_GENERATOR_SIERPINSKI_CARPET];
	_initialise();
}

IteratedFunctionSystem :: ~IteratedFunctionSystem()
{
	_cleanup();
	m_mapCells.clear();
	m_pGenerator = NULL;
	for (int i = 0 ; i < eIFS_GENERATOR_COUNT ; i++)
	{
		delete m_arrayGenerators[i];
	}
}

void IteratedFunctionSystem :: SwapBuffers()
{
	ProceduralGenerator::SwapBuffers();
}

void IteratedFunctionSystem :: SetGenerator( eIFS_GENERATOR p_eGenerator)
{
	m_initialised = true;
	ClearAllThreads();

	m_pGenerator = m_arrayGenerators[p_eGenerator];

	_initialise();
}

void IteratedFunctionSystem :: _initialise()
{
	m_mapCells.clear();
	m_lstCells.clear();
	m_pGenerator->Initialise( m_mapCells, m_lstCells, m_VctCells);
	m_backBuffer.init();
	IteratedFunctionSystem::Thread * l_pThread;

	for (size_t i = 0 ; i < m_pGenerator->GetInitCellsMapSize() ; i++)
	{
		l_pThread = CreateThread<IteratedFunctionSystem::Thread>( m_sizeImage.x, 0, m_sizeImage.y, m_sizeImage.y, UbPixel());
		l_pThread->Initialise( & (m_pGenerator->GetInitCellsMap( i)), & (m_pGenerator->GetInitCellsLst( i)), & (m_pGenerator->GetInitCellsVct( i)), & m_backBuffer, & m_pGenerator);
		l_pThread->Run();
	}

	m_initialised = true;
}

//*************************************************************************************************

IMPLEMENT_PLUGIN( IfsPlugin, IfsPanel, IteratedFunctionSystem)

IfsPlugin :: IfsPlugin()
{
}

IfsPlugin :: ~IfsPlugin()
{
}

wxString IfsPlugin :: GetName()
{
	return wxT( "Iterated Function System");
}

//*************************************************************************************************

IfsPanel :: IfsPanel( wxWindow * p_pWindow, wxWindowID p_id, const wxPoint & p_ptPosition, const wxSize & p_size)
	:	ConfigPanel( p_pWindow, p_id, p_ptPosition, p_size)
{
}

IfsPanel :: ~IfsPanel()
{
}

void IfsPanel :: Create( IteratedFunctionSystem * p_pGenerator, wxWindow * p_pParent, const wxPoint & p_ptPosition, const wxSize & p_size)
{
	SetSize( p_ptPosition.x, p_ptPosition.y, p_size.x, p_size.y);
	Reparent( p_pParent);
	m_pGenerator = p_pGenerator;

	new wxStaticText( this, wxID_ANY, wxT( "Angle :"), wxPoint( 20, 0), wxSize( p_size.x - 60, 20));
	wxString l_strValue;
	l_strValue << m_pGenerator->GetAngle();
	m_pEditAngle = new wxTextCtrl( this, eAngle, l_strValue, wxPoint( 20, 20), wxSize( p_size.x - 40, 20), wxTE_PROCESS_ENTER);

	l_strValue.clear();
	new wxStaticText( this, wxID_ANY, wxT( "Shift :"), wxPoint( 20, 40), wxSize( p_size.x - 60, 20));
	l_strValue << m_pGenerator->GetShift();
	m_pEditShift = new wxTextCtrl( this, eShift, l_strValue, wxPoint( 20, 60), wxSize( p_size.x - 40, 20), wxTE_PROCESS_ENTER);

	l_strValue.clear();
	new wxStaticText( this, wxID_ANY, wxT( "Size :"), wxPoint( 20, 80), wxSize( p_size.x - 60, 20));
	l_strValue << m_pGenerator->GetSize();
	m_pEditSize = new wxTextCtrl( this, eSize, l_strValue, wxPoint( 20, 100), wxSize( p_size.x - 40, 20), wxTE_PROCESS_ENTER);

	new wxStaticText( this, wxID_ANY, wxT( "Generator :"), wxPoint( 20, 120), wxSize( p_size.x - 60, 20));
	wxArrayString l_arrayChoices;

	for (int i = 0 ; i < IteratedFunctionSystem::eIFS_GENERATOR_COUNT ; i++)
	{
		l_arrayChoices.push_back( IteratedFunctionSystem::ArrayNames[i]);
	}

	m_pComboGenerator = new wxComboBox( this, eGenerator, IteratedFunctionSystem::ArrayNames[IteratedFunctionSystem::eIFS_GENERATOR_SIERPINSKI_CARPET], wxPoint( 20, 140), wxSize( p_size.x - 40, 25), l_arrayChoices, wxCB_READONLY);
	m_pComboGenerator->SetFont( GetFont());
}

BEGIN_EVENT_TABLE( IfsPanel, ConfigPanel)
	EVT_TEXT_ENTER(	eAngle,		IfsPanel::_onAngle)
	EVT_TEXT_ENTER(	eShift,		IfsPanel::_onShift)
	EVT_TEXT_ENTER(	eSize,		IfsPanel::_onSizeIfs)
	EVT_COMBOBOX(	eGenerator,	IfsPanel::_onGenerator)
END_EVENT_TABLE()

void IfsPanel :: _onAngle( wxCommandEvent & p_event)
{
	int l_iValue = atoi( p_event.GetString().char_str());
	m_pGenerator->SetAngle( l_iValue);
}

void IfsPanel :: _onShift( wxCommandEvent & p_event)
{
	int l_iValue = atoi( p_event.GetString().char_str());
	m_pGenerator->SetShift( l_iValue);
}

void IfsPanel :: _onSizeIfs( wxCommandEvent & p_event)
{
	int l_iValue = atoi( p_event.GetString().char_str());
	m_pGenerator->SetSize( l_iValue);
}

void IfsPanel :: _onGenerator( wxCommandEvent & p_event)
{
	m_pGenerator->SetGenerator( IteratedFunctionSystem::eIFS_GENERATOR( p_event.GetInt()));
}

//*************************************************************************************************
