#include "LibGenerator/PrecompiledHeader.h"

#include "LibGenerator/Thread.h"

#ifdef __WXMSW__
#	include<wx/msw/msvcrt.h>
#endif

using namespace ProceduralTextures;

//*********************************************************************************************

Thread :: Thread()
	:	m_pThread( NULL)
	,	m_bSuspended( false)
	,	m_dwID( 0)
{
}

Thread :: Thread( PDWThreadStartFunction lpStartAddress, void * lpParameter, void * lpThreadAttributes, size_t dwStackSize, eTHREAD_STATE eCreationFlags)
	:	m_pThread( NULL)
	,	m_bSuspended( false)
	,	m_dwID( 0)
{
	CreateThread( lpStartAddress, lpParameter, lpThreadAttributes, dwStackSize, eCreationFlags);
}

Thread :: Thread( PUIThreadStartFunction lpStartAddress, void * lpParameter, void * lpThreadAttributes, size_t dwStackSize, eTHREAD_STATE eCreationFlags)
	:	m_pThread( NULL)
	,	m_bSuspended( false)
	,	m_dwID( 0)
{
	CreateThread( lpStartAddress, lpParameter, lpThreadAttributes, dwStackSize, eCreationFlags);
}

Thread :: ~Thread()
{
	delete m_pThread;
}

void Thread :: CreateThread( PDWThreadStartFunction lpStartAddress, void * lpParameter, void * lpThreadAttributes, size_t dwStackSize, eTHREAD_STATE eCreationFlags)
{
	m_bSuspended = (eCreationFlags & eCreateSuspended ? true : false);
	m_pThread = new boost::thread( PG_THREAD_FUNCTOR( lpStartAddress, lpParameter));
}

void Thread :: CreateThread( PUIThreadStartFunction lpStartAddress, void * lpParameter, void * lpThreadAttributes, size_t dwStackSize, eTHREAD_STATE eCreationFlags)
{
	m_bSuspended = (eCreationFlags & eCreateSuspended ? true : false);
	m_pThread = new boost::thread( PG_THREAD_FUNCTOR( lpStartAddress, lpParameter));
}

bool Thread :: Wait( unsigned long dwMilliseconds)
{
	bool l_bReturn = true;

	if (m_pThread != NULL)
	{
		boost::posix_time::time_duration time = boost::posix_time::microsec( dwMilliseconds);
		l_bReturn = m_pThread->timed_join( time);

		if (l_bReturn)
		{
			delete m_pThread;
			m_pThread = NULL;
		}
	}

	return l_bReturn;
}

void Thread :: Kill()
{
	if (m_pThread != NULL)
	{
		m_pThread->join();
		delete m_pThread;
		m_pThread = NULL;
	}
}

//*********************************************************************************************

Thread * ProceduralTextures :: CreateThread( PDWThreadStartFunction lpStartAddress, void * lpParameter, void * lpThreadAttributes, size_t dwStackSize, eTHREAD_STATE dwCreationFlags, unsigned long * lpThreadId)
{
	Thread * l_pReturn = new Thread;
	l_pReturn->CreateThread( lpStartAddress, lpParameter, lpThreadAttributes, dwStackSize, dwCreationFlags);

	if (lpThreadId != NULL)
	{
		*lpThreadId = l_pReturn->GetID();
	}

	return l_pReturn;
}

Thread * ProceduralTextures :: CreateThread( PUIThreadStartFunction lpStartAddress, void * lpParameter, void * lpThreadAttributes, size_t dwStackSize, eTHREAD_STATE dwCreationFlags, unsigned long * lpThreadId)
{
	Thread * l_pReturn = new Thread;
	l_pReturn->CreateThread( lpStartAddress, lpParameter, lpThreadAttributes, dwStackSize, dwCreationFlags);

	if (lpThreadId != NULL)
	{
		*lpThreadId = l_pReturn->GetID();
	}

	return l_pReturn;
}

bool ProceduralTextures :: WaitForMultipleThreads( bool bWaitAll, unsigned long dwMilliseconds, size_t p_uiNbThreads, ...)
{
	bool l_bReturn = true;
	std::vector <boost::thread *> l_arrayHandles;
	va_list l_vaArgs;
	va_start( l_vaArgs, p_uiNbThreads);

	for (size_t i = 0 ; i < p_uiNbThreads ; i++)
	{
		ProceduralTextures::Thread * l_pThread = va_arg( l_vaArgs, ProceduralTextures::Thread *);
		l_arrayHandles.push_back( l_pThread->GetHandle());
	}

	va_end( l_vaArgs);

	boost::posix_time::time_duration time = boost::posix_time::microsec( dwMilliseconds);

	for (size_t i = 0 ; i < l_arrayHandles.size() && l_bReturn ; i++)
	{
		l_bReturn = l_arrayHandles[i]->timed_join( time);
	}

	return l_bReturn;
}

bool ProceduralTextures :: WaitForSingleThread( Thread * pThread, unsigned long dwMilliseconds)
{
	return pThread->Wait( dwMilliseconds);
}
