#include "LibGenerator/PrecompiledHeader.h"

#include "LibGenerator/Vertex.h"

#ifdef __WXMSW__
#	include<wx/msw/msvcrt.h>
#endif

using namespace ProceduralTextures;

//*************************************************************************************************

unsigned long long Vertex::Count = 0;
size_t Vertex::Size = 11;
BufferElementDeclaration Vertex::VertexDeclarationElements[] =
{
	{ 0, eUsagePosition,	eType3Floats},
	{ 0, eUsageNormal,		eType3Floats},
	{ 0, eUsageTangent,		eType3Floats},
	{ 0, eUsageTexCoords0,	eType2Floats}
};

BufferDeclaration Vertex::VertexDeclaration( VertexDeclarationElements);

Vertex :: Vertex( float x, float y, float z, size_t p_index)
	:	BufferElement( & VertexDeclaration)
	,	m_uiIndex( p_index)
	,	m_bManual( false)
{
	GetElement<Point3f>( 0)[0] = x;
	GetElement<Point3f>( 0)[1] = y;
	GetElement<Point3f>( 0)[2] = z;
	Count++;
	UnlinkCoords();
}

Vertex :: Vertex( const Vertex & p_vertex)
	:	BufferElement( p_vertex)
	,	m_bManual( p_vertex.m_bManual)
	,	m_uiIndex( p_vertex.GetIndex())
{
	Count++;
	UnlinkCoords();

}

Vertex :: Vertex( const Point3f & p_point)
	:	BufferElement( & VertexDeclaration)
	,	m_uiIndex( 0)
	,	m_bManual( false)
{
	GetElement<Point3f>( 0) = p_point;
	Count++;
	UnlinkCoords();
}

Vertex :: Vertex( float * p_pBuffer)
	:	BufferElement( & VertexDeclaration)
	,	m_uiIndex( 0)
	,	m_bManual( false)
{
	Count++;
	m_pBuffer = (unsigned char *)p_pBuffer;
	_link();
}

Vertex :: Vertex( const Vertex & p_v1, const Vertex & p_v2)
	:	BufferElement( & VertexDeclaration)
	,	m_uiIndex( 0)
	,	m_bManual( false)
{
	GetElement<Point3f>( 0)[0] = p_v2.GetCoords()[0] - p_v1.GetCoords()[0];
	GetElement<Point3f>( 0)[1] = p_v2.GetCoords()[1] - p_v1.GetCoords()[1];
	GetElement<Point3f>( 0)[2] = p_v2.GetCoords()[2] - p_v1.GetCoords()[2];
	Count++;
	UnlinkCoords();
}

Vertex :: ~Vertex()
{
	Count--;
}

void Vertex :: LinkCoords( float * p_pBuffer)
{
	BufferElement::LinkCoords( p_pBuffer);
}

void Vertex :: UnlinkCoords()
{
	BufferElement::UnlinkCoords();
}

Vertex & Vertex :: operator =( const Vertex & p_vertex)
{
	BufferElement::operator =( p_vertex);
	m_uiIndex		= p_vertex.m_uiIndex;
	m_bManual		= p_vertex.m_bManual;
	return * this;
}

void Vertex :: SetNormal( const Point3f & val, bool p_bManual)
{
	SetNormal( val[0], val[1], val[2], p_bManual);
}

void Vertex :: SetNormal( float x, float y, float z, bool p_bManual)
{
	GetElement<Point3f>( 1)[0] = x;
	GetElement<Point3f>( 1)[1] = y;
	GetElement<Point3f>( 1)[2] = z;

	if (p_bManual || m_bManual)
	{
		p_bManual = true;
	}

	m_bManual = p_bManual;
}

void Vertex :: SetNormal( const float * p_pCoords, bool p_bManual)
{
	SetNormal( p_pCoords[0], p_pCoords[1], p_pCoords[2], p_bManual);
}

void Vertex :: SetTexCoord( const Point2f & val, bool p_bManual)
{
	SetTexCoord( val[0], val[1], p_bManual);
}

void Vertex :: SetTexCoord( float x, float y, bool p_bManual)
{
	GetElement<Point2f>( 3)[0] = x;
	GetElement<Point2f>( 3)[1] = y;

	if (p_bManual || m_bManual)
	{
		p_bManual = true;
	}

	m_bManual = p_bManual;
}

void Vertex :: SetTexCoord( const float * p_pCoords, bool p_bManual)
{
	SetTexCoord( p_pCoords[0], p_pCoords[1], p_bManual);
}

void Vertex :: SetTangent( const Point3f & val, bool p_bManual)
{
	SetTangent( val[0], val[1], val[2], p_bManual);
}

void Vertex :: SetTangent( float x, float y, float z, bool p_bManual)
{
	GetElement<Point3f>( 2)[0] = x;
	GetElement<Point3f>( 2)[1] = y;
	GetElement<Point3f>( 2)[2] = z;

	if (p_bManual || m_bManual)
	{
		p_bManual = true;
	}

	m_bManual = p_bManual;
}

void Vertex :: SetTangent( const float * p_pCoords, bool p_bManual)
{
	SetTangent( p_pCoords[0], p_pCoords[1], p_pCoords[2], p_bManual);
}

void Vertex :: _link()
{
	BufferElement::_link();
}