#include "ProceduralGenerator/PrecompiledHeader.h"

#include "ProceduralGenerator/MainFrame.h"
#include "ProceduralGenerator/Application.h"
#include "ProceduralGenerator/ProjectListDialog.h"
#include "ProceduralGenerator/RenderPanel.h"
#include "ProceduralGenerator/StatusBar.h"
#include "../resource/ProceduralGenerator/exit.xpm"
#include "../resource/ProceduralGenerator/print_screen.xpm"
#include "../resource/ProceduralGenerator/select.xpm"

#ifdef __WXMSW__
#	include<wx/msw/msvcrt.h>
#endif

using namespace ProceduralTextures;

//*************************************************************************************************

IMPLEMENT_CLASS( MainFrame, wxFrame);

MainFrame :: MainFrame()
	:	wxFrame( NULL, eMyID, wxT( "Procedural Generator"), wxPoint( 0, 0), wxDefaultSize, wxRESIZE_BORDER | wxMAXIMIZE_BOX | wxMINIMIZE_BOX | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
	,	m_pRenderPanel	( NULL)
	,	m_pConfigPanel	( NULL)
	,	m_pPlugin		( NULL)
	,	m_pStatusBar	( NULL)
{
	SetBackgroundColour( * wxBLACK);
	SetForegroundColour( * wxWHITE);
	m_pStatusBar = new StatusBar( this);
	SetStatusBar( m_pStatusBar);
}

MainFrame :: ~MainFrame()
{
}

void MainFrame :: Initialise()
{
	wxSize l_size( 256, 256);
	SetClientSize( l_size);
	_loadPlugins();
	_buildMenuBar();
	int l_attribList[] = { WX_GL_RGBA, WX_GL_LEVEL, 0, WX_GL_DOUBLEBUFFER };
	m_pRenderPanel = new RenderPanel( this, l_attribList, wxPoint( 0, 0), l_size);
	m_pRenderPanel->SetBackgroundColour( GetBackgroundColour());
	m_pRenderPanel->SetForegroundColour( GetForegroundColour());
	Show();
}

void MainFrame :: _buildMenuBar()
{
	wxPNGHandler * l_pHandler = new wxPNGHandler;
	wxImage::AddHandler( l_pHandler);
	wxToolBar * l_pToolbar = CreateToolBar( wxTB_FLAT | wxTB_HORIZONTAL);
	wxMemoryInputStream l_isSelect( select_png, sizeof( select_png));
	wxImage l_imgSelect( l_isSelect, wxBITMAP_TYPE_PNG);
	wxMemoryInputStream l_isPrint( print_screen_png, sizeof( print_screen_png));
	wxImage l_imgPrint( l_isPrint, wxBITMAP_TYPE_PNG);
	wxMemoryInputStream l_isExit( exit_png, sizeof( exit_png));
	wxImage l_imgExit( l_isExit, wxBITMAP_TYPE_PNG);
	l_pToolbar->SetBackgroundColour( GetBackgroundColour());
	l_pToolbar->SetForegroundColour( GetForegroundColour());
	l_pToolbar->AddTool( eSelectGenerator,	wxT( "Select Generator"),	l_imgSelect.Rescale( 32, 32, wxIMAGE_QUALITY_HIGH));
	l_pToolbar->AddTool( ePrintScreen,		wxT( "Print screen"),		l_imgPrint.Rescale( 32, 32, wxIMAGE_QUALITY_HIGH));
	l_pToolbar->AddTool( eExit,				wxT( "Exit"),				l_imgExit.Rescale( 32, 32, wxIMAGE_QUALITY_HIGH));
	wxImage::RemoveHandler( l_pHandler->GetName());
	l_pToolbar->Realize();
}

ProceduralGenerator * MainFrame :: _selectGenerator()
{
	ProceduralGenerator * l_pReturn = NULL;
	wxArrayString l_choices;

	for (std::map<wxString, PluginBase *>::iterator l_it = m_mapPluginsByName.begin() ; l_it != m_mapPluginsByName.end() ; ++l_it)
	{
		l_choices.push_back( l_it->first);
	}

	ProjectListDialog l_dialog( this, l_choices);

	if (l_dialog.ShowModal() == wxID_OK)
	{
		wxString l_strProjectName = l_dialog.GetProject();
		l_pReturn = _loadGenerator( l_strProjectName, l_dialog.GetResolution());
	}

	return l_pReturn;
}

ProceduralGenerator * MainFrame :: _loadGenerator( const wxString & p_strName, const wxSize & p_size)
{
	ProceduralGenerator * l_pReturn = NULL;
	std::map<wxString, PluginBase *>::iterator l_it = m_mapPluginsByName.find( p_strName);

	if (l_it != m_mapPluginsByName.end())
	{
		m_pPlugin = l_it->second;
		SetClientSize( p_size);
		l_pReturn = l_it->second->CreateGenerator( p_size.x, p_size.y, MainFrame::eMyID, this);

		if (l_pReturn->NeedsConfigPanel())
		{
			l_it->second->CreateConfigPanel( this);
			SetClientSize( p_size.x + 150, p_size.y);
			l_it->second->InitConfigPanel( wxPoint( p_size.x, 0), wxSize( 150, p_size.y));
			m_pConfigPanel = l_it->second->GetConfigPanel();
		}

		SetLabel( p_strName);
	}

	return l_pReturn;
}

void MainFrame :: _destroyGenerator()
{
	if (m_pPlugin != NULL)
	{
		m_pPlugin->DestroyGenerator();
		m_pPlugin->DestroyConfigPanel();
	}

	m_pConfigPanel = NULL;
}

void MainFrame :: _loadPlugins()
{
	wxStandardPathsBase & l_paths = wxStandardPaths::Get();
	wxFileName l_dataDir = l_paths.GetExecutablePath();
	wxString l_strDataDir = l_dataDir.GetPath() + wxT( "/Plugins/");
	wxDir l_dir( l_strDataDir);
	wxString l_strFileName;

	if (l_dir.IsOpened())
	{
		bool l_bRes = l_dir.GetFirst( & l_strFileName, wxString( wxT( "*")) + wxDynamicLibrary::GetDllExt(), wxDIR_FILES);

		while (l_bRes)
		{
			wxDynamicLibrary * l_pDll = new wxDynamicLibrary;
			l_pDll->Load( l_strDataDir + l_strFileName);
			Sleep( 100);

			if (l_pDll->HasSymbol( wxT( "CreatePlugin")))
			{
				wxDYNLIB_FUNCTION( PCreatePluginFunction, CreatePlugin, * l_pDll);

				if (pfnCreatePlugin != NULL)
				{
					PluginBase * l_pPlugin = pfnCreatePlugin( l_pDll);

					if (l_pPlugin != NULL)
					{
						m_mapPluginsByName.insert( std::pair<wxString, PluginBase *>( l_pPlugin->GetName(), l_pPlugin));
					}
				}
			}

			l_bRes = l_dir.GetNext( & l_strFileName);
		}
	}
}

void MainFrame :: _close()
{
	Hide();

	if (m_pRenderPanel != NULL)
	{
		m_pRenderPanel->Close();
		delete m_pRenderPanel;
		m_pRenderPanel = NULL;
	}

	_destroyGenerator();

	for (std::map<wxString, PluginBase *>::iterator l_it = m_mapPluginsByName.begin() ; l_it != m_mapPluginsByName.end() ; ++l_it)
	{
		l_it->second->Destroy();
	}

	m_mapPluginsByName.clear();
}

BEGIN_EVENT_TABLE( MainFrame, wxFrame)
	EVT_SIZE(								MainFrame::_onSize)
	EVT_CLOSE(								MainFrame::_onClose)
	EVT_STATUS_UPDATE(	eMyID,				MainFrame::_onStatusUpdate)
	EVT_KEY_UP(								MainFrame::_onKeyUp)
	EVT_MENU( 			eExit,				MainFrame::_onExit)
	EVT_MENU( 			eSelectGenerator,	MainFrame::_onSelectGenerator)
	EVT_MENU( 			ePrintScreen,		MainFrame::_onPrintScreen)
END_EVENT_TABLE()

void MainFrame :: _onSize( wxSizeEvent & p_event)
{
	wxSize sizeClient = GetClientSize();

	if (m_pConfigPanel != NULL)
	{
		sizeClient.x -= m_pConfigPanel->GetClientSize().x;
	}

	if (m_pRenderPanel != NULL)
	{
		m_pRenderPanel->SetSize( 0, 0, sizeClient.x, sizeClient.y, wxSIZE_FORCE);
	}

	if (m_pConfigPanel != NULL)
	{
		m_pConfigPanel->SetSize( sizeClient.x, 0, 150, sizeClient.y, wxSIZE_FORCE);
	}

	p_event.Skip();
}

void MainFrame :: _onClose( wxCloseEvent & p_event)
{
	_close();
	p_event.Skip();
}

void MainFrame :: _onStatusUpdate( wxStatusUpdateEvent & p_event)
{
	wxMilliClock_t l_diff = p_event.GetTime();
	wxString l_strStatus;

	if (p_event.GetIndex() == 0)
	{
		l_strStatus = wxT( "C : ");
	}
	else
	{
		l_strStatus = wxT( "R : ");
	}

	if (l_diff != 0)
	{
		std::basic_stringstream<wxChar> l_streamMs;
		std::basic_stringstream<wxChar> l_streamFps;
		l_streamMs.width( 5);
		l_streamMs.fill( wxT( '0'));
		l_streamMs << std::right << l_diff.ToLong();
		l_streamFps.width( 4);
		l_streamFps.fill( wxT( '0'));
		l_streamFps << std::right << (wxMilliClock_t( 1000) / l_diff).ToLong();
		m_pStatusBar->SetStatusText( l_strStatus + l_streamMs.str() + wxT( "ms - ") + l_streamFps.str() + wxT( " FPS"), p_event.GetIndex());
	}
}

void MainFrame :: _onKeyUp( wxKeyEvent & p_event)
{
	if (m_pRenderPanel != NULL && p_event.GetKeyCode() == WXK_F12)
	{
		m_pRenderPanel->SaveFrame();
	}
}

void MainFrame :: _onExit( wxCommandEvent & p_event)
{
	Close( true);
}

void MainFrame :: _onSelectGenerator( wxCommandEvent & p_event)
{
	m_pRenderPanel->SetGenerator( NULL);
	_destroyGenerator();
	ProceduralGenerator * l_pGenerator = _selectGenerator();
	Maximize( false);
	m_pRenderPanel->SetGenerator( l_pGenerator);
	p_event.Skip();
}

void MainFrame :: _onPrintScreen( wxCommandEvent & p_event)
{
	if (m_pRenderPanel != NULL)
	{
		m_pRenderPanel->SaveFrame();
	}
	p_event.Skip();
}

//*************************************************************************************************
