#include "ProceduralGenerator/PrecompiledHeader.h"

#include "ProceduralGenerator/RenderPanel.h"

#ifdef __WXMSW__
#	include<wx/msw/msvcrt.h>
#endif

using namespace ProceduralTextures;

//*************************************************************************************************

RenderPanel :: RenderPanel( wxWindow * p_parent, int * p_attribList, const wxPoint & p_position, const wxSize & p_size)
	:	wxGLCanvas( p_parent, wxID_ANY, p_attribList, p_position, p_size)
	,	m_pContext			( NULL)
	,	m_bInitialised		( false)
	,	m_pEvent			( new wxStatusUpdateEvent( p_parent->GetId()))
	,	m_pGenerator		( NULL)
	,	m_event				( 1, p_parent->GetId())
	,	m_pTimer 			( new wxTimer( this, m_iTimerId))
{
	SetBackgroundColour( p_parent->GetBackgroundColour());
	SetForegroundColour( p_parent->GetForegroundColour());
	Show();
}

RenderPanel :: ~RenderPanel()
{
	OpenGl::Cleanup();
}

void RenderPanel :: SetGenerator( ProceduralGenerator * p_pGenerator)
{
	m_pTimer->Stop();

	if (m_pGenerator != NULL && m_pContext != NULL)
	{
		m_pContext->SetCurrent( * this);
		m_pGenerator->GlCleanup();
	}

	m_pGenerator = p_pGenerator;

	if (m_pGenerator != NULL)
	{
		SetSize( p_pGenerator->GetDisplaySize());
		m_pTimer->Start( 1);
	}

	m_bInitialised = false;
}

void RenderPanel :: _initGL()
{
	if (m_pContext == NULL)
	{
		m_pContext = new wxGLContext( this, NULL);
	}

	m_pContext->SetCurrent( * this);
	m_pGenerator->GlInit();
	m_bInitialised = true;
}

void RenderPanel :: _render( bool p_bChanged)
{
	if (IsShown() && GetParent()->IsShown())
	{
		if (m_pGenerator != NULL && m_pGenerator->IsInitialised())
		{
			if (m_pContext == NULL || ! m_bInitialised)
			{
				_initGL();
			}

			wxClientDC l_dc( this);
			m_pContext->SetCurrent( * this);

			m_pGenerator->GlPreRender();
			m_pGenerator->GlRender( p_bChanged);
			m_pGenerator->GlPostRender();

			SwapBuffers();
		}
		else
		{
			wxClientDC l_dc( this);
			l_dc.DrawRectangle( wxPoint( 0, 0), GetClientSize());
		}
	}
}

void RenderPanel :: SaveFrame()
{
	if (m_pGenerator != NULL)
	{
		m_pGenerator->SaveFrame();
	}
}

BEGIN_EVENT_TABLE( RenderPanel, wxGLCanvas)
	EVT_SIZE(				RenderPanel::_onSize)
	EVT_TIMER( m_iTimerId,	RenderPanel::_onTimer)
	EVT_CLOSE(				RenderPanel::_onClose)
	EVT_KEY_UP(				RenderPanel::_onKeyUp)
END_EVENT_TABLE()

void RenderPanel :: _onSize( wxSizeEvent & p_event)
{
	if (m_pGenerator)
	{
		m_pGenerator->Resize( p_event.GetSize());
	}
	p_event.Skip();
}

void RenderPanel :: _onTimer( wxTimerEvent & p_event)
{
	if (m_pGenerator != NULL)
	{
		bool l_bChanged = m_pGenerator->Step();
		m_event.ResetTime();
		_render( l_bChanged);
		m_event.ResetTime();
		GetParent()->GetEventHandler()->AddPendingEvent( m_event);
	}
	else
	{
		wxClientDC l_dc( this);
		l_dc.DrawRectangle( wxPoint( 0, 0), GetClientSize());
	}
}

void RenderPanel :: _onClose( wxCloseEvent & p_event)
{
	if (m_pTimer != NULL)
	{
		delete m_pTimer;
	}

	if (m_pContext != NULL && m_pGenerator != NULL)
	{
		wxClientDC l_dc( this);
		m_pContext->SetCurrent( * this);
		m_pGenerator->GlCleanup();
	}

	if (m_pContext != NULL)
	{
		delete m_pContext;
	}

	if (m_pEvent != NULL)
	{
		delete m_pEvent;
	}

	p_event.Skip();
}

void RenderPanel :: _onKeyUp( wxKeyEvent & p_event)
{
	if (p_event.GetKeyCode() == WXK_F12)
	{
		SaveFrame();
	}
}
